package nckd.jimin.jyyy.fi.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.entity.plugin.args.BeginOperationTransactionArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import nckd.jimin.jyyy.fi.common.FinanceCommonUtils;

import java.util.Date;
import java.util.List;

/**
 * 表单标识：外办单(用车申请单nckd_er_dailyvehicleb_ext)
 * @author wanghaiwu_kd
 * @date 2025/04/29
 */
public class DailyVehicleBillSubmitOpPlugin extends AbstractOperationServicePlugIn {
    private static final Log logger = LogFactory.getLog(DailyVehicleBillSubmitOpPlugin.class);

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);

        List<String> fieldKeys = e.getFieldKeys();

        fieldKeys.add("applier");
        fieldKeys.add("startdate");
        fieldKeys.add("enddate");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);

        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] dataEntitys = this.getDataEntities();
                for (ExtendedDataEntity dataEntity : dataEntitys) {
                    if(dataEntity == null){
                        continue;
                    }
                    DynamicObject vehicleReqBillInfo = dataEntity.getDataEntity();

                    Date startDate = vehicleReqBillInfo.getDate("startdate");
                    Date endDate = vehicleReqBillInfo.getDate("enddate");

                    if(startDate == null || endDate == null){
                        this.addErrorMessage(dataEntity, "开始时间、结束时间不能为空!");
                    }

                    Long billId = vehicleReqBillInfo.getLong("id");
                    Long userId = vehicleReqBillInfo.getDynamicObject("applier").getLong("id");

                    String checkMsg = FinanceCommonUtils.checkDateIsExists(billId, userId, startDate, endDate);

                    if(checkMsg.length() > 0) {
                        this.addErrorMessage(dataEntity, "存在开始时间、结束时间重叠的单据!" + checkMsg);
                    }
                }
            }
        });
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
    }

    @Override
    public void beginOperationTransaction(BeginOperationTransactionArgs e) {
        super.beginOperationTransaction(e);
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
    }
}
