package nckd.jimin.jyyy.fi.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.entity.plugin.args.BeginOperationTransactionArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import java.util.Date;
import java.util.List;

import nckd.jimin.jyyy.fi.common.FinanceCommonUtils;

/**
 * 表单标识：出差申请单(nckd_er_tripreqbill_ext)
 * @author wanghaiwu_kd
 * @date 2025/04/29
 */
public class TripReqBillSubmitOpPlugin extends AbstractOperationServicePlugIn {
    private static final Log logger = LogFactory.getLog(TripReqBillSubmitOpPlugin.class);

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);

        List<String> fieldKeys = e.getFieldKeys();

//        fieldKeys.add("tripentry");
        fieldKeys.add("applier");
        fieldKeys.add("tripentry.startdate");
        fieldKeys.add("tripentry.enddate");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);

        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] dataEntitys = this.getDataEntities();
                for (ExtendedDataEntity dataEntity : dataEntitys) {
                    if(dataEntity == null){
                        continue;
                    }
                    DynamicObject tripReqInfo = dataEntity.getDataEntity();

                    DynamicObjectCollection tripentry = tripReqInfo.getDynamicObjectCollection("tripentry");

                    for(DynamicObject entry : tripentry){
                        Date startDate = entry.getDate("startdate");
                        Date endDate = entry.getDate("enddate");

                        if(startDate == null || endDate == null){
                            this.addErrorMessage(dataEntity, "开始时间、结束时间不能为空!");
                            continue;
                        }

                        Long billId = tripReqInfo.getLong("id");
                        Long userId = tripReqInfo.getDynamicObject("applier").getLong("id");

                        String checkMsg = FinanceCommonUtils.checkDateIsExists(billId, userId, startDate, endDate);

                        if(checkMsg.length() > 0) {
                            this.addErrorMessage(dataEntity, "存在开始时间、结束时间重叠的单据!" + checkMsg);
                        }
                    }
                }
            }
        });
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
    }

    @Override
    public void beginOperationTransaction(BeginOperationTransactionArgs e) {
        super.beginOperationTransaction(e);
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
    }
}
