package nckd.jimin.jyyy.fi.task;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.exception.KDException;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.api.StopTask;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import nckd.base.helper.CommonHelperUtils;
import nckd.jimin.jyyy.fi.plugin.operate.SRMHelperUtils;
import java.math.BigDecimal;
import java.util.*;


/**
 * 付款状态回写srm调度任务类
 * @author wanghaiwu_kd
 * @date 2025/05/15
 */
public class WriteBackPayResult2SRMTask extends AbstractTask implements StopTask {
    private static final Log logger = LogFactory.getLog(WriteBackPayResult2SRMTask.class);
    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
        //对公报销单回写
        writeBackPublicBill(requestContext, map);
        //预付单回写
        writeBackPrepayBill(requestContext, map);



    }

    private void writeBackPublicBill(RequestContext requestContext, Map<String, Object> map){
        String entityName = "er_publicreimbursebill";
        String pattern = "yyyy-MM-dd HH:mm:ss";

        //srm状态，1：SRM已推送；2：已退回SRM；3：已反写SRM；4：反写SRM失败
        List<String> srmStatusList = Arrays.asList("2", "3");

        QFilter filter = new QFilter("billstatus", QCP.equals, "G");
        filter.and(new QFilter("nckd_srmstatus", QCP.not_in, srmStatusList));
        filter.and(new QFilter("nckd_srmbillno", QCP.is_notnull, null));
        filter.and(new QFilter("nckd_srmbillno", QCP.not_equals2, " "));
        filter.and(new QFilter("nckd_srmbillno", QCP.not_equals2, ""));
//        filter.and(new QFilter("billno", QCP.equals, "DGBX-250411-0021"));

        DynamicObjectType type = EntityMetadataCache.getDataEntityType(entityName);
        //先找到批量的pkid
        List<Object> list = QueryServiceHelper.queryPrimaryKeys(entityName,  filter.toArray(), null, Integer.MAX_VALUE);
        //根据pkid找到完整的对象
        DynamicObject[] billEntities = BusinessDataServiceHelper.load(list.toArray(), type);

        List<DynamicObject> listObj = new ArrayList<>();
        for (DynamicObject payBillEntity : billEntities) {
            //srm单据号
            String billno = payBillEntity.getString("billno");
            String srmBillNo = payBillEntity.getString("nckd_srmbillno");
            String paymentDate = "";

            if(payBillEntity.getDate("head_paydate") != null){
                paymentDate = CommonHelperUtils.getDateFormatString(payBillEntity.getDate("head_paydate"), pattern);
            }

            BigDecimal paymentAmount = BigDecimal.ZERO;
            DynamicObjectCollection accountEntrys = payBillEntity.getDynamicObjectCollection("accountentry");

            if(accountEntrys != null && accountEntrys.size() > 0){
                for(DynamicObject entry : accountEntrys){
                    paymentAmount.add(entry.getBigDecimal("orireceiveamount"));
                }
            }

            StringBuffer errMessage = new StringBuffer();
            Map<String, String> returnMap = SRMHelperUtils.writeBackPayResulst(srmBillNo, paymentDate, paymentAmount);

            if(returnMap != null){
                if("1".equals(returnMap.get("code"))){
                    if(errMessage.length() > 0){
                        errMessage.append(",");
                    }
                    logger.info("对公报销单(" + billno + ", " + srmBillNo + ")回写失败，" + returnMap.get("msg"));
                    payBillEntity.set("nckd_srmstatus", "4");
                } else {
                    payBillEntity.set("nckd_srmstatus", "3");
                }

                listObj.add(payBillEntity);
                if(listObj.size() == 500) {
                    SaveServiceHelper.update(listObj.toArray(new DynamicObject[]{}));
                    listObj.clear();
                }
            }
        }
        if(listObj.size() > 0) {
            SaveServiceHelper.update(listObj.toArray(new DynamicObject[]{}));
        }
    }

    private void writeBackPrepayBill(RequestContext requestContext, Map<String, Object> map){
        String entityName = "er_prepaybill";
        String pattern = "yyyy-MM-dd HH:mm:ss";

        //srm状态，1：SRM已推送；2：已退回SRM；3：已反写SRM；4：反写SRM失败
        List<String> srmStatusList = Arrays.asList("2", "3");

        QFilter filter = new QFilter("billstatus", QCP.equals, "G");
        filter.and(new QFilter("nckd_srmstatus", QCP.not_in, srmStatusList));
        filter.and(new QFilter("nckd_srmbillno", QCP.is_notnull, null));
        filter.and(new QFilter("nckd_srmbillno", QCP.not_equals2, " "));
        filter.and(new QFilter("nckd_srmbillno", QCP.not_equals2, ""));
//        filter.and(new QFilter("billno", QCP.equals, "PV-202410-006562"));

        DynamicObjectType type = EntityMetadataCache.getDataEntityType(entityName);
        //先找到批量的pkid
        List<Object> list = QueryServiceHelper.queryPrimaryKeys(entityName,  filter.toArray(), null, Integer.MAX_VALUE);
        //根据pkid找到完整的对象
        DynamicObject[] billEntities = BusinessDataServiceHelper.load(list.toArray(), type);

        List<DynamicObject> listObj = new ArrayList<>();
        for (DynamicObject payBillEntity : billEntities) {
            //srm单据号
            String billno = payBillEntity.getString("billno");
            String srmBillNo = payBillEntity.getString("nckd_srmbillno");
            String paymentDate = "";

            if(payBillEntity.getDate("head_paydate") != null){
                paymentDate = CommonHelperUtils.getDateFormatString(payBillEntity.getDate("head_paydate"), pattern);
            }

            BigDecimal paymentAmount = BigDecimal.ZERO;
            DynamicObjectCollection accountEntrys = payBillEntity.getDynamicObjectCollection("accountentry");

            if(accountEntrys != null && accountEntrys.size() > 0){
                for(DynamicObject entry : accountEntrys){
                    paymentAmount.add(entry.getBigDecimal("orireceiveamount"));
                }
            }

            StringBuffer errMessage = new StringBuffer();
            Map<String, String> returnMap = SRMHelperUtils.writeBackPayResulst(srmBillNo, paymentDate, paymentAmount);

            if(returnMap != null){
                if("1".equals(returnMap.get("code"))){
                    if(errMessage.length() > 0){
                        errMessage.append(",");
                    }
                    logger.info("预付单(" + billno + ", " + srmBillNo + ")回写失败，" + returnMap.get("msg"));
                    payBillEntity.set("nckd_srmstatus", "4");
                } else {
                    payBillEntity.set("nckd_srmstatus", "3");
                }

                listObj.add(payBillEntity);
                if(listObj.size() == 500) {
                    SaveServiceHelper.update(listObj.toArray(new DynamicObject[]{}));
                    listObj.clear();
                }
            }
        }
        if(listObj.size() > 0) {
            SaveServiceHelper.update(listObj.toArray(new DynamicObject[]{}));
        }
    }
}
